@@ -1,12 +1,12 @@
 use anyhow::Result;
 use console::Style;
-use once_cell::sync::Lazy;
 use std::{
     fmt::{self, Display},
+    sync::LazyLock,
     time::Duration,
 };
 
-use crate::formatters::color::{color, BOLD_GREEN, BOLD_PINK, BOLD_YELLOW, DIM, NORMAL};
+use crate::formatters::color::{BOLD_GREEN, BOLD_PINK, BOLD_YELLOW, DIM, NORMAL, color};
 use crate::{formatters::get_response_formatter, options, stats::ResponseStats};
 
 use super::StatsFormatter;
@@ -37,7 +37,7 @@ impl Display for CompactResponseStats {
 
         let response_formatter = get_response_formatter(&self.mode);
 
-        for (source, responses) in &stats.error_map {
+        for (source, responses) in super::sort_stat_map(&stats.error_map) {
             color!(f, BOLD_YELLOW, "[{}]:\n", source)?;
             for response in responses {
                 writeln!(
@@ -47,9 +47,16 @@ impl Display for CompactResponseStats {
                 )?;
             }
 
-            if let Some(suggestions) = &stats.suggestion_map.get(source) {
-                writeln!(f, "\n\u{2139} Suggestions")?;
-                for suggestion in *suggestions {
+            if let Some(suggestions) = stats.suggestion_map.get(source) {
+                // Sort suggestions
+                let mut sorted_suggestions: Vec<_> = suggestions.iter().collect();
+                sorted_suggestions.sort_by(|a, b| {
+                    let (a, b) = (a.to_string().to_lowercase(), b.to_string().to_lowercase());
+                    numeric_sort::cmp(&a, &b)
+                });
+
+                writeln!(f, "\nℹ Suggestions")?;
+                for suggestion in sorted_suggestions {
                     writeln!(f, "{suggestion}")?;
                 }
             }
@@ -73,6 +80,8 @@ impl Display for CompactResponseStats {
         write_if_any(stats.unknown, "❓", "Unknown", &BOLD_PINK, f)?;
         write_if_any(stats.excludes, "👻", "Excluded", &BOLD_YELLOW, f)?;
         write_if_any(stats.timeouts, "⏳", "Timeouts", &BOLD_YELLOW, f)?;
+        write_if_any(stats.unsupported, "⛔", "Unsupported", &BOLD_YELLOW, f)?;
+        write_if_any(stats.redirects, "🔀", "Redirects", &BOLD_YELLOW, f)?;
 
         Ok(())
     }
@@ -82,7 +91,7 @@ fn write_if_any(
     value: usize,
     symbol: &str,
     text: &str,
-    style: &Lazy<Style>,
+    style: &LazyLock<Style>,
     f: &mut fmt::Formatter<'_>,
 ) -> Result<(), fmt::Error> {
     if value > 0 {
@@ -159,6 +168,7 @@ mod tests {
             duration_secs: 0,
             error_map,
             suggestion_map: HashMap::default(),
+            redirect_map: HashMap::default(),
             unsupported: 0,
             redirects: 0,
             cached: 0,
@@ -178,8 +188,10 @@ mod tests {
         assert!(result.contains("🚫 2 Errors"));
 
         assert!(result.contains("[https://example.com/]:"));
-        assert!(result
-            .contains("https://github.com/mre/idiomatic-rust-doesnt-exist-man | 404 Not Found"));
+        assert!(
+            result
+                .contains("https://github.com/mre/idiomatic-rust-doesnt-exist-man | 404 Not Found")
+        );
         assert!(result.contains("https://github.com/mre/boom | 500 Internal Server Error"));
     }
 }
