@@ -14,7 +14,7 @@
 //! ```
 //!
 //! For more specific use-cases you can build a lychee client yourself,
-//! using the `ClientBuilder` which can be used to
+//! using the [`ClientBuilder`] which can be used to
 //! configure and run your own link checker and grants full flexibility:
 //!
 //! ```
@@ -50,6 +50,8 @@
 #[cfg(doctest)]
 doc_comment::doctest!("../../README.md");
 
+/// Check online archives to try and restore broken links
+pub mod archive;
 mod basic_auth;
 pub mod chain;
 mod checker;
@@ -71,11 +73,6 @@ pub mod remap;
 /// local IPs or e-mail addresses
 pub mod filter;
 
-/// Test utilities
-#[cfg(test)]
-#[macro_use]
-pub mod test_utils;
-
 #[cfg(test)]
 use doc_comment as _; // required for doctest
 use ring as _; // required for apple silicon
@@ -90,15 +87,15 @@ pub use crate::{
     chain::{ChainResult, Handler},
     // Constants get exposed so that the CLI can use the same defaults as the library
     client::{
-        check, Client, ClientBuilder, DEFAULT_MAX_REDIRECTS, DEFAULT_MAX_RETRIES,
-        DEFAULT_RETRY_WAIT_TIME_SECS, DEFAULT_TIMEOUT_SECS, DEFAULT_USER_AGENT,
+        Client, ClientBuilder, DEFAULT_MAX_REDIRECTS, DEFAULT_MAX_RETRIES,
+        DEFAULT_RETRY_WAIT_TIME_SECS, DEFAULT_TIMEOUT_SECS, DEFAULT_USER_AGENT, check,
     },
     collector::Collector,
     filter::{Excludes, Filter, Includes},
     types::{
-        uri::valid::Uri, AcceptRange, AcceptRangeError, Base, BasicAuthCredentials,
-        BasicAuthSelector, CacheStatus, CookieJar, ErrorKind, FileType, Input, InputContent,
-        InputSource, Request, Response, ResponseBody, Result, Status, StatusCodeExcluder,
-        StatusCodeSelector,
+        AcceptRange, AcceptRangeError, Base, BasicAuthCredentials, BasicAuthSelector, CacheStatus,
+        CookieJar, ErrorKind, FileExtensions, FileType, Input, InputContent, InputResolver,
+        InputSource, Redirects, Request, ResolvedInputSource, Response, ResponseBody, Result,
+        Status, StatusCodeExcluder, StatusCodeSelector, uri::valid::Uri,
     },
 };
