@@ -1,3 +1,4 @@
+use http::StatusCode;
 use serde::{Serialize, Serializer};
 use std::error::Error;
 use std::hash::Hash;
@@ -7,14 +8,15 @@ use tokio::task::JoinError;
 
 use super::InputContent;
 use crate::types::StatusCodeSelectorError;
-use crate::{basic_auth::BasicAuthExtractorError, utils, Uri};
+use crate::{Uri, basic_auth::BasicAuthExtractorError, utils};
 
 /// Kinds of status errors
 /// Note: The error messages can change over time, so don't match on the output
 #[derive(Error, Debug)]
 #[non_exhaustive]
 pub enum ErrorKind {
-    /// Network error while handling request
+    /// Network error while handling request.
+    /// This does not include erroneous status codes, `RejectedStatusCode` will be used in that case.
     #[error("Network error")]
     NetworkRequest(#[source] reqwest::Error),
     /// Cannot read the body of the received response
@@ -64,10 +66,14 @@ pub enum ErrorKind {
     #[error("Cannot find file")]
     InvalidFilePath(Uri),
 
-    /// The given URI cannot be converted to a file path
+    /// The given URI's fragment could not be found within the page content
     #[error("Cannot find fragment")]
     InvalidFragment(Uri),
 
+    /// Cannot resolve local directory link using the configured index files
+    #[error("Cannot find index file within directory")]
+    InvalidIndexFile(Vec<String>),
+
     /// The given path cannot be converted to a URI
     #[error("Invalid path to URL conversion: {0}")]
     InvalidUrlFromPath(PathBuf),
@@ -119,7 +125,9 @@ pub enum ErrorKind {
     InvalidGlobPattern(#[from] glob::PatternError),
 
     /// The GitHub API could not be called because of a missing GitHub token.
-    #[error("GitHub token not specified. To check GitHub links reliably, use `--github-token` flag / `GITHUB_TOKEN` env var.")]
+    #[error(
+        "GitHub token not specified. To check GitHub links reliably, use `--github-token` flag / `GITHUB_TOKEN` env var."
+    )]
     MissingGitHubToken,
 
     /// Used an insecure URI where a secure variant was reachable
@@ -142,14 +150,14 @@ pub enum ErrorKind {
     #[error("Invalid status code: {0}")]
     InvalidStatusCode(u16),
 
+    /// The given status code was not accepted (this depends on the `accept` configuration)
+    #[error(r#"Rejected status code (this depends on your "accept" configuration)"#)]
+    RejectedStatusCode(StatusCode),
+
     /// Regex error
     #[error("Error when using regex engine: {0}")]
     Regex(#[from] regex::Error),
 
-    /// Too many redirects (HTTP 3xx) were encountered (configurable)
-    #[error("Too many redirects")]
-    TooManyRedirects(#[source] reqwest::Error),
-
     /// Basic auth extractor error
     #[error("Basic auth extractor error")]
     BasicAuthExtractorError(#[from] BasicAuthExtractorError),
@@ -161,6 +169,12 @@ pub enum ErrorKind {
     /// Status code selector parse error
     #[error("Status code range error")]
     StatusCodeSelectorError(#[from] StatusCodeSelectorError),
+
+    /// Test-only error variant for formatter tests
+    /// Available in both test and debug builds to support cross-crate testing
+    #[cfg(any(test, debug_assertions))]
+    #[error("Generic test error")]
+    TestError,
 }
 
 impl ErrorKind {
@@ -170,36 +184,157 @@ impl ErrorKind {
     /// request type. The output is purely meant for humans (e.g. for status
     /// messages) and future changes are expected.
     #[must_use]
+    #[allow(clippy::too_many_lines)]
     pub fn details(&self) -> Option<String> {
         match self {
             ErrorKind::NetworkRequest(e) => {
-                if let Some(status) = e.status() {
-                    Some(
+                        // Get detailed, actionable error analysis
+                        Some(utils::reqwest::analyze_error_chain(e))
+                    }
+            ErrorKind::RejectedStatusCode(status) => Some(
                         status
                             .canonical_reason()
                             .unwrap_or("Unknown status code")
                             .to_string(),
-                    )
-                } else {
-                    // Get the relevant details from the specific reqwest error
-                    let details = utils::reqwest::trim_error_output(e);
-
-                    // Provide support for common error types
-                    if e.is_connect() {
-                        Some(format!("{details} Maybe a certificate error?"))
-                    } else {
-                        Some(details)
-                    }
-                }
-            }
+                    ),
             ErrorKind::GithubRequest(e) => {
-                if let octocrab::Error::GitHub { source, .. } = &**e {
-                    Some(source.message.clone())
-                } else {
-                    None
-                }
-            }
-            _ => self.source().map(ToString::to_string),
+                        if let octocrab::Error::GitHub { source, .. } = &**e {
+                            Some(source.message.clone())
+                        } else {
+                            // Fall back to generic error analysis
+                            Some(e.to_string())
+                        }
+                    }
+            ErrorKind::InvalidFilePath(_uri) => Some(
+                "File not found. Check if file exists and path is correct".to_string()
+            ),
+            ErrorKind::ReadFileInput(e, path) => match e.kind() {
+                        std::io::ErrorKind::NotFound => Some(
+                            "Check if file path is correct".to_string()
+                        ),
+                        std::io::ErrorKind::PermissionDenied => Some(format!(
+                            "Permission denied: '{}'. Check file permissions",
+                            path.display()
+                        )),
+                        std::io::ErrorKind::IsADirectory => Some(format!(
+                            "Path is a directory, not a file: '{}'. Check file path",
+                            path.display()
+                        )),
+                        _ => Some(format!("File read error for '{}': {}", path.display(), e)),
+                    },
+            ErrorKind::ReadStdinInput(e) => match e.kind() {
+                        std::io::ErrorKind::UnexpectedEof => {
+                            Some("Stdin input ended unexpectedly. Check input data".to_string())
+                        }
+                        std::io::ErrorKind::InvalidData => {
+                            Some("Invalid data from stdin. Check input format".to_string())
+                        }
+                        _ => Some(format!("Stdin read error: {e}")),
+                    },
+            ErrorKind::ParseUrl(_, url) => {
+                        Some(format!("Invalid URL format: '{url}'. Check URL syntax"))
+                    }
+            ErrorKind::EmptyUrl => {
+                        Some("Empty URL found. Check for missing links or malformed markdown".to_string())
+                    }
+            #[cfg(any(test, debug_assertions))]
+            ErrorKind::TestError => Some("Test error for formatter testing".to_string()),
+            ErrorKind::InvalidFile(path) => Some(format!(
+                        "Invalid file path: '{}'. Check if file exists and is readable",
+                        path.display()
+                    )),
+            ErrorKind::ReadResponseBody(error) => Some(format!(
+                "Failed to read response body: {error}. Server may have sent invalid data",
+            )),
+            ErrorKind::BuildRequestClient(error) => Some(format!(
+                "Failed to create HTTP client: {error}. Check system configuration",
+            )),
+            ErrorKind::RuntimeJoin(join_error) => Some(format!(
+                "Task execution failed: {join_error}. Internal processing error"
+            )),
+            ErrorKind::Utf8(_utf8_error) => Some(
+                "Invalid UTF-8 sequence found. File contains non-UTF-8 characters".to_string()
+            ),
+            ErrorKind::BuildGithubClient(error) => Some(format!(
+                "Failed to create GitHub client: {error}. Check token and network connectivity",
+            )),
+            ErrorKind::InvalidGithubUrl(url) => Some(format!(
+                "Invalid GitHub URL format: '{url}'. Check URL syntax",
+            )),
+            ErrorKind::InvalidFragment(_uri) => Some(
+                "Fragment not found in document. Check if fragment exists or page structure".to_string()
+            ),
+            ErrorKind::InvalidUrlFromPath(path_buf) => Some(format!(
+                "Cannot convert path to URL: '{}'. Check path format",
+                path_buf.display()
+            )),
+            ErrorKind::UnreachableEmailAddress(uri, reason) => Some(format!(
+                "Email address unreachable: '{uri}'. {reason}",
+            )),
+            ErrorKind::InvalidHeader(invalid_header_value) => Some(format!(
+                "Invalid HTTP header: {invalid_header_value}. Check header format",
+            )),
+            ErrorKind::InvalidBase(base, reason) => Some(format!(
+                "Invalid base URL or directory: '{base}'. {reason}",
+            )),
+            ErrorKind::InvalidBaseJoin(text) => Some(format!(
+                "Cannot join '{text}' with base URL. Check relative path format",
+            )),
+            ErrorKind::InvalidPathToUri(path) => Some(format!(
+                "Cannot convert path to URI: '{path}'. Check path format",
+            )),
+            ErrorKind::RootDirMustBeAbsolute(path_buf) => Some(format!(
+                "Root directory must be absolute: '{}'. Use full path",
+                path_buf.display()
+            )),
+            ErrorKind::UnsupportedUriType(uri_type) => Some(format!(
+                "Unsupported URI type: '{uri_type}'. Only http, https, file, and mailto are supported",
+            )),
+            ErrorKind::InvalidUrlRemap(remap) => Some(format!(
+                "Invalid URL remapping: '{remap}'. Check remapping syntax",
+            )),
+            ErrorKind::DirTraversal(error) => Some(format!(
+                "Directory traversal failed: {error}. Check directory permissions",
+            )),
+            ErrorKind::InvalidGlobPattern(pattern_error) => Some(format!(
+                "Invalid glob pattern: {pattern_error}. Check pattern syntax",
+            )),
+            ErrorKind::MissingGitHubToken => Some(
+                "GitHub token required. Use --github-token flag or GITHUB_TOKEN environment variable".to_string()
+            ),
+            ErrorKind::InsecureURL(uri) => Some(format!(
+                "Insecure HTTP URL detected: use '{}' instead of HTTP",
+                uri.as_str().replace("http://", "https://")
+            )),
+            ErrorKind::Channel(_send_error) => Some(
+                "Internal communication error. Processing thread failed".to_string()
+            ),
+            ErrorKind::InvalidUrlHost => Some(
+                "URL missing hostname. Check URL format".to_string()
+            ),
+            ErrorKind::InvalidURI(uri) => Some(format!(
+                "Invalid URI format: '{uri}'. Check URI syntax",
+            )),
+            ErrorKind::InvalidStatusCode(code) => Some(format!(
+                "Invalid HTTP status code: {code}. Must be between 100-999",
+            )),
+            ErrorKind::Regex(error) => Some(format!(
+                "Regular expression error: {error}. Check regex syntax",
+            )),
+            ErrorKind::BasicAuthExtractorError(basic_auth_extractor_error) => Some(format!(
+                "Basic authentication error: {basic_auth_extractor_error}. Check credentials format",
+            )),
+            ErrorKind::Cookies(reason) => Some(format!(
+                "Cookie handling error: {reason}. Check cookie file format",
+            )),
+            ErrorKind::StatusCodeSelectorError(status_code_selector_error) => Some(format!(
+                "Status code selector error: {status_code_selector_error}. Check accept configuration",
+            )),
+            ErrorKind::InvalidIndexFile(index_files) => match &index_files[..] {
+                [] => "No directory links are allowed because index_files is defined and empty".to_string(),
+                [name] => format!("An index file ({name}) is required"),
+                [init @ .., tail] => format!("An index file ({}, or {}) is required", init.join(", "), tail),
+            }.into()
         }
     }
 
@@ -262,9 +397,6 @@ impl PartialEq for ErrorKind {
             (Self::Regex(e1), Self::Regex(e2)) => e1.to_string() == e2.to_string(),
             (Self::DirTraversal(e1), Self::DirTraversal(e2)) => e1.to_string() == e2.to_string(),
             (Self::Channel(_), Self::Channel(_)) => true,
-            (Self::TooManyRedirects(e1), Self::TooManyRedirects(e2)) => {
-                e1.to_string() == e2.to_string()
-            }
             (Self::BasicAuthExtractorError(e1), Self::BasicAuthExtractorError(e2)) => {
                 e1.to_string() == e2.to_string()
             }
@@ -272,10 +404,14 @@ impl PartialEq for ErrorKind {
             (Self::InvalidFile(p1), Self::InvalidFile(p2)) => p1 == p2,
             (Self::InvalidFilePath(u1), Self::InvalidFilePath(u2)) => u1 == u2,
             (Self::InvalidFragment(u1), Self::InvalidFragment(u2)) => u1 == u2,
+            (Self::InvalidIndexFile(p1), Self::InvalidIndexFile(p2)) => p1 == p2,
             (Self::InvalidUrlFromPath(p1), Self::InvalidUrlFromPath(p2)) => p1 == p2,
             (Self::InvalidBase(b1, e1), Self::InvalidBase(b2, e2)) => b1 == b2 && e1 == e2,
             (Self::InvalidUrlRemap(r1), Self::InvalidUrlRemap(r2)) => r1 == r2,
             (Self::EmptyUrl, Self::EmptyUrl) => true,
+            (Self::RejectedStatusCode(c1), Self::RejectedStatusCode(c2)) => c1 == c2,
+            #[cfg(any(test, debug_assertions))]
+            (Self::TestError, Self::TestError) => true,
 
             _ => false,
         }
@@ -309,6 +445,7 @@ impl Hash for ErrorKind {
             Self::Utf8(e) => e.to_string().hash(state),
             Self::InvalidFilePath(u) => u.hash(state),
             Self::InvalidFragment(u) => u.hash(state),
+            Self::InvalidIndexFile(p) => p.hash(state),
             Self::UnreachableEmailAddress(u, ..) => u.hash(state),
             Self::InsecureURL(u, ..) => u.hash(state),
             Self::InvalidBase(base, e) => (base, e).hash(state),
@@ -320,12 +457,16 @@ impl Hash for ErrorKind {
             Self::InvalidHeader(e) => e.to_string().hash(state),
             Self::InvalidGlobPattern(e) => e.to_string().hash(state),
             Self::InvalidStatusCode(c) => c.hash(state),
+            Self::RejectedStatusCode(c) => c.hash(state),
             Self::Channel(e) => e.to_string().hash(state),
             Self::MissingGitHubToken | Self::InvalidUrlHost => {
                 std::mem::discriminant(self).hash(state);
             }
+            #[cfg(any(test, debug_assertions))]
+            Self::TestError => {
+                std::mem::discriminant(self).hash(state);
+            }
             Self::Regex(e) => e.to_string().hash(state),
-            Self::TooManyRedirects(e) => e.to_string().hash(state),
             Self::BasicAuthExtractorError(e) => e.to_string().hash(state),
             Self::Cookies(e) => e.to_string().hash(state),
             Self::StatusCodeSelectorError(e) => e.to_string().hash(state),
