@@ -9,6 +9,7 @@ mod error;
 mod file;
 mod input;
 pub(crate) mod mail;
+pub(crate) mod redirect_history;
 mod request;
 pub(crate) mod resolver;
 mod response;
@@ -23,7 +24,8 @@ pub use cache::CacheStatus;
 pub use cookies::CookieJar;
 pub use error::ErrorKind;
 pub use file::{FileExtensions, FileType};
-pub use input::{Input, InputContent, InputSource};
+pub use input::{Input, InputContent, InputResolver, InputSource, ResolvedInputSource};
+pub use redirect_history::Redirects;
 pub use request::Request;
 pub use response::{Response, ResponseBody};
 pub use status::Status;
