@@ -3,7 +3,7 @@ use std::fmt::Display;
 use http::StatusCode;
 use serde::Serialize;
 
-use crate::{InputSource, Status, Uri};
+use crate::{ResolvedInputSource, Status, Uri};
 
 /// Response type returned by lychee after checking a URI
 //
@@ -14,13 +14,13 @@ use crate::{InputSource, Status, Uri};
 // `pub(crate)` is insufficient, because the `stats` module is in the `bin`
 // crate crate.
 #[derive(Debug)]
-pub struct Response(InputSource, pub ResponseBody);
+pub struct Response(ResolvedInputSource, pub ResponseBody);
 
 impl Response {
     #[inline]
     #[must_use]
     /// Create new response
-    pub const fn new(uri: Uri, status: Status, source: InputSource) -> Self {
+    pub const fn new(uri: Uri, status: Status, source: ResolvedInputSource) -> Self {
         Response(source, ResponseBody { uri, status })
     }
 
@@ -35,7 +35,7 @@ impl Response {
     #[must_use]
     /// Retrieve the underlying source of the response
     /// (e.g. the input file or the URL)
-    pub const fn source(&self) -> &InputSource {
+    pub const fn source(&self) -> &ResolvedInputSource {
         &self.0
     }
 
@@ -62,9 +62,9 @@ impl Serialize for Response {
     }
 }
 
+/// Encapsulates the state of a URI check
 #[allow(clippy::module_name_repetitions)]
 #[derive(Debug, Serialize, Hash, PartialEq, Eq)]
-/// Encapsulates the state of a URI check
 pub struct ResponseBody {
     #[serde(flatten)]
     /// The URI which was checked
